import logging
import math
import csv
import click

from collections import defaultdict
from compendium_v2.environment import setup_logging
from compendium_v2 import db, survey_db
from compendium_v2.background_task import xlsx_to_csv_sheet_parsing_task
from compendium_v2.config import load
from compendium_v2.db import model
from compendium_v2.survey_db import model as survey_model

setup_logging()

logger = logging.getLogger('survey_publisher_v1')


def init_db(config):
    dsn_prn = config['SQLALCHEMY_DATABASE_URI']
    db.init_db_model(dsn_prn)
    dsn_survey = config['SURVEY_DATABASE_URI']
    survey_db.init_db_model(dsn_survey)


def db_budget_migration():
    with survey_db.session_scope() as survey_session, \
            db.session_scope() as session:
        _entries = session.query(model.BudgetEntry)

        inserted = defaultdict(dict)

        for entry in _entries:
            inserted[entry.nren][entry.year] = entry.budget

        data = survey_session.query(survey_model.Nrens)
        for nren in data:
            for budget in nren.budgets:
                abbrev = nren.abbreviation
                year = budget.year

                if float(budget.budget) > 200:
                    logger.info(
                        f'Incorrect Data: '
                        f'{abbrev} has budget set to '
                        f'>200M EUR for {year}. ({budget.budget})')

                budget_entry = model.BudgetEntry(
                    nren=abbrev, budget=budget.budget, year=year)
                session.merge(budget_entry)

        # Import the data to database
        xlsx_to_csv_sheet_parsing_task.parse_budget_xlsx_file()
        with open('compendium_v2/background_task/csv/BudgetCsvFile.csv',
                  newline='') as csvfile:
            reader = csv.reader(csvfile)

            for row in reader:
                if row is not None:
                    budget_entry = model.BudgetEntry(
                        nren=row[0], budget=row[1], year=row[2])
                    session.merge(budget_entry)
        session.commit()


def db_funding_migration():
    with db.session_scope() as session:

        # Import the data to database
        xlsx_to_csv_sheet_parsing_task.parse_income_source_xlsx_file()
        with open('compendium_v2/background_task/csv/FundingSourceCsvFile.csv',
                  newline='') as csvfile:
            reader = csv.reader(csvfile)

            for row in reader:
                if row is not None:
                    _data = [float(row[2]), float(row[3]), float(row[4]),
                             float(row[5]), float(row[6])]

                    total = sum(_data)
                    if not math.isclose(total, 100, abs_tol=0.01):
                        logger.info(
                            f'{row[0]} funding sources for {row[1]}'
                            f' do not sum to 100% ({total})')

                    budget_entry = model.FundingSource(
                        nren=row[0], year=row[1],
                        client_institutions=row[2],
                        european_funding=row[3],
                        gov_public_bodies=row[4],
                        commercial=row[5],
                        other=row[6])
                    session.merge(budget_entry)
        session.commit()


@click.command()
@click.option('--config', type=click.STRING, default='config.json')
def cli(config):
    app_config = load(open(config, 'r'))
    print("survey-publisher-v1 starting")
    init_db(app_config)
    db_budget_migration()
    db_funding_migration()


if __name__ == "__main__":
    cli()
