import contextlib
import logging
from typing import Optional, Union, Callable, Iterator

from sqlalchemy import create_engine
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import sessionmaker, Session

logger = logging.getLogger(__name__)
_SESSION_MAKER: Union[None, sessionmaker] = None


@contextlib.contextmanager
def session_scope(
        callback_before_close: Optional[Callable] = None) -> Iterator[Session]:
    # best practice is to keep session scope separate from data processing
    # cf. https://docs.sqlalchemy.org/en/13/orm/session_basics.html

    assert _SESSION_MAKER
    session = _SESSION_MAKER()
    try:
        yield session
        session.commit()
        if callback_before_close:
            callback_before_close()
    except SQLAlchemyError:
        logger.error('caught sql layer exception, rolling back')
        session.rollback()
        raise  # re-raise, will be handled by main consumer
    finally:
        session.close()


def postgresql_dsn(db_username, db_password, db_hostname, db_name, port=5432):
    return (f'postgresql://{db_username}:{db_password}'
            f'@{db_hostname}:{port}/{db_name}')


def init_db_model(dsn):
    global _SESSION_MAKER

    # cf. https://docs.sqlalchemy.org/en
    #        /latest/orm/extensions/automap.html
    engine = create_engine(dsn, pool_size=10)
    _SESSION_MAKER = sessionmaker(bind=engine)
